using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace WindowsGame1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        IndexBuffer indexBuffer;
        VertexBuffer vertexBuffer;
        VertexDeclaration vertexDeclaration;
        Matrix world = Matrix.CreateRotationX(MathHelper.ToRadians(-10.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(-15.0f));
        Effect effect;

        readonly float turningSpeed = MathHelper.ToRadians(90.0f); // Radiant pro Sekunde

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            Window.AllowUserResizing = true;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            indexBuffer = new IndexBuffer(graphics.GraphicsDevice, typeof(ushort), 6, BufferUsage.WriteOnly);
            indexBuffer.SetData<ushort>(new ushort[] { 0, 1, 2, 0, 2, 3 });

            vertexBuffer = new VertexBuffer(graphics.GraphicsDevice, typeof(VertexPositionNormalTexture), 4, BufferUsage.WriteOnly);
            vertexBuffer.SetData<VertexPositionNormalTexture>
                (
                    new VertexPositionNormalTexture[]
                    {
                        new VertexPositionNormalTexture(new Vector3(0.0f, 0.0f, 0.0f), new Vector3(0.0f, 0.0f, 1.0f), new Vector2(0.0f, 0.0f)),
                        new VertexPositionNormalTexture(new Vector3(1.0f, 0.0f, 0.0f), new Vector3(0.0f, 0.0f, 1.0f), new Vector2(1.0f, 0.0f)),
                        new VertexPositionNormalTexture(new Vector3(1.0f, 1.0f, 0.0f), new Vector3(0.0f, 0.0f, 1.0f), new Vector2(1.0f, 1.0f)),
                        new VertexPositionNormalTexture(new Vector3(0.0f, 1.0f, 0.0f), new Vector3(0.0f, 0.0f, 1.0f), new Vector2(0.0f, 1.0f))
                    }
                );

            vertexDeclaration = new VertexDeclaration(graphics.GraphicsDevice,
                VertexPositionNormalTexture.VertexElements);

            effect = Content.Load<Effect>("effect");
            effect.CurrentTechnique = effect.Techniques["phong"];
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            indexBuffer.Dispose();
            vertexBuffer.Dispose();
            effect.Dispose();
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            if (Keyboard.GetState().IsKeyDown(Keys.Left))
            {
                world = world * Matrix.CreateRotationY((float)(-turningSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Right))
            {
                world = world * Matrix.CreateRotationY((float)(turningSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Up))
            {
                world = world * Matrix.CreateRotationX((float)(-turningSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Down))
            {
                world = world * Matrix.CreateRotationX((float)(turningSpeed * gameTime.ElapsedGameTime.TotalSeconds));
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            // also show the backfaces of the triangles
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;

            // uncomment this to show the triangles
            //graphics.GraphicsDevice.RenderState.FillMode = FillMode.WireFrame;

            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            graphics.GraphicsDevice.Indices = indexBuffer;
            graphics.GraphicsDevice.Vertices[0].SetSource(vertexBuffer, 0, VertexPositionNormalTexture.SizeInBytes);
            graphics.GraphicsDevice.VertexDeclaration = vertexDeclaration;

            Matrix view = Matrix.CreateLookAt(new Vector3(0.0f, 0.0f, 10.0f), Vector3.Zero, Vector3.Up);
            Matrix projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(50.0f),
                graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height,
                0.1f, 100.0f);

            effect.Parameters["WorldViewProj"].SetValue(world * view * projection);
            effect.Parameters["World"].SetValue(world);
            effect.Parameters["WorldIT"].SetValue(Matrix.Transpose(Matrix.Invert(world)));
            effect.Parameters["ViewI"].SetValue(Matrix.Invert(view));
            effect.Parameters["DirToLight"].SetValue(Vector3.Normalize(new Vector3(-1.0f, 1.0f, 1.0f)));

            effect.Begin();
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();
                graphics.GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, 4, 0, 2);
                pass.End();
            }
            effect.End();

            base.Draw(gameTime);
        }
    }
}